SystemOrganization addCategory: #'Magritte-Pharo-Model'!

MAFileModel subclass: #MAExternalFileModel
	instanceVariableNames: 'location'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Pharo-Model'!

MAExternalFileModel class
	instanceVariableNames: 'baseDirectory baseUrl'!

!MAExternalFileModel commentStamp: 'lr 9/28/2007 08:11' prior: 0!
I manage the file-data I represent on the file-system. From the programmer this looks the same as if the file would be in memory (==*MAMemoryFileModel*==), as it is transparently loaded and written out as necessary.

- The ==#baseDirectory== is the place where Magritte puts its file-database. Keep this value to nil to make it default to a subdirectory next to the Squeak image.
- The ==#baseUrl== is a nice optimization to allow Apache (or any other Web Server) to directly serve the files. ==#baseUrl== is an absolute URL-prefix that is used to generate the path to the file. If you have specified one the file data does not go trough the image anymore, but instead is directly served trough the properly configured Web Server.

The files are currently stored using the following scheme:

=/files/9d/bsy8kyp45g0q7blphknk48zujap2wd/earthmap1k.jpg
=1     2   3                              4

#Is the #baseDirectory as specified in the settings.
#Are 256 directories named '00' to 'ff' to avoid having thousands of files in the same directory. Unfortunately this leads to problems with the Squeak file primitives and some filesystems don't handle that well. This part is generated at random.
#This is a secure id, similar to the Seaside session key. It is generated at random and provides a security system that even works trough Apache (you have to disable directory listings of course): if you don't know the file-name you cannot access the file.
#This is the original file-name. Subclasses might want to store other cached versions of the same file there, for example resized images, etc.!

MAExternalFileModel class
	instanceVariableNames: 'baseDirectory baseUrl'!

!MAExternalFileModel classSide methodsFor: 'accessing' stamp: 'lr 9/28/2007 07:43'!
baseDirectory
	^ baseDirectory ifNil: [ FileDirectory default directoryNamed: 'files' ]! !

!MAExternalFileModel classSide methodsFor: 'accessing' stamp: 'lr 9/28/2007 07:42'!
baseDirectory: aStringOrDirectory
	"Defines the base-directory where the files are stored. If this value is set to nil, it default to a subdirectory of of the current image-location."

	baseDirectory := aStringOrDirectory isString
		ifTrue: [ FileDirectory on: aStringOrDirectory ]
		ifFalse: [ aStringOrDirectory ]! !

!MAExternalFileModel classSide methodsFor: 'accessing' stamp: 'jf 9/28/2009 13:36'!
baseDirectoryPath
	^ self baseDirectory pathName! !

!MAExternalFileModel classSide methodsFor: 'accessing' stamp: 'jf 9/28/2009 13:33'!
baseDirectoryPath: aString
	self baseDirectory: (FileDirectory on: aString)! !

!MAExternalFileModel classSide methodsFor: 'accessing' stamp: 'lr 9/28/2007 07:44'!
baseUrl
	^ baseUrl! !

!MAExternalFileModel classSide methodsFor: 'accessing' stamp: 'lr 9/28/2007 07:42'!
baseUrl: aString
	"Defines the base-URL where the files are served from, when using an external web server. This setting is left to nil by default, causing the files to be served trough the image."

	baseUrl := aString isNil ifFalse: [
		aString last = $/
			ifFalse: [ aString ]
			ifTrue: [ aString copyUpToLast: $/ ] ]! !

!MAExternalFileModel classSide methodsFor: 'public' stamp: 'lr 9/9/2008 00:02'!
garbageCollect
	"Remove obsolete files from the file-system that do not have a counterpart in memory anymore. This method has to be called manually and is not intended to be portable."
	
	| mark sweep |
	mark := self allInstances
		collect: [ :each | each directory pathName ].
	sweep := Array with: self baseDirectory.
	self locationDefinition do: [ :definition |
		sweep := sweep gather: [ :directory |
			directory entries isEmpty
				ifTrue: [ directory recursiveDelete ].
			(directory directoryNames
				select: [ :each | each size = definition first ])
				collect: [ :each | directory directoryNamed: each ] ] ].
	sweep do: [ :directory |
		(mark includes: directory pathName)
			ifFalse: [ directory recursiveDelete ] ]! !

!MAExternalFileModel classSide methodsFor: 'initialization' stamp: 'lr 9/28/2007 08:20'!
initialize
	baseDirectory := baseUrl := nil! !

!MAExternalFileModel classSide methodsFor: 'initialization' stamp: 'lr 9/8/2008 23:26'!
locationDefinition
	^ #( ( 2 '63450af8d9c2e17b' ) ( 30 'iaojv41bw67e0tud5m9rgplqfy8x3cs2kznh' ) )! !

!MAExternalFileModel methodsFor: 'comparing' stamp: 'lr 11/6/2009 15:38'!
= anObject
    ^ super = anObject and: [ self location = anObject location ]! !

!MAExternalFileModel methodsFor: 'configuration' stamp: 'lr 9/28/2007 07:43'!
baseDirectory
	^ self class baseDirectory! !

!MAExternalFileModel methodsFor: 'configuration' stamp: 'lr 9/28/2007 07:45'!
baseUrl
	^ self class baseUrl! !

!MAExternalFileModel methodsFor: 'accessing' stamp: 'lr 10/18/2007 18:21'!
contents
	| stream |
	^ (self directory exists and: [ self directory fileExists: self filename ])
		ifFalse: [ ByteArray new ]
		ifTrue: [
			stream := self readStream.
			[ stream contents ]
				ensure: [ stream close ] ]! !

!MAExternalFileModel methodsFor: 'accessing' stamp: 'nfr 10/17/2007 12:26'!
contents: aByteArray
	| stream |
	stream := self writeStream.
	[ stream nextPutAll: aByteArray asByteArray ]
		ensure: [ stream close ].
	super contents: aByteArray! !

!MAExternalFileModel methodsFor: 'accessing-dynamic' stamp: 'lr 9/28/2007 07:44'!
directory
	^ self location
		inject: self baseDirectory
		into: [ :result :each | result directoryNamed: each ]! !

!MAExternalFileModel methodsFor: 'initialization' stamp: 'lr 11/18/2007 10:34'!
finalize
	| directory |
	directory := self directory.
	directory exists
		ifTrue: [ directory recursiveDelete ].
	[ (directory := directory containingDirectory) entries isEmpty ]
		whileTrue: [ directory recursiveDelete ].
	super finalize.
	location := nil! !

!MAExternalFileModel methodsFor: 'comparing' stamp: 'lr 11/6/2009 15:38'!
hash
	^ super hash bitXor: self location hash! !

!MAExternalFileModel methodsFor: 'accessing-dynamic' stamp: 'lr 1/26/2007 18:19'!
location
	^ location ifNil: [ location := self uniqueLocation: self locationDefinition ]! !

!MAExternalFileModel methodsFor: 'configuration' stamp: 'lr 9/8/2008 23:26'!
locationDefinition
	^ self class locationDefinition! !

!MAExternalFileModel methodsFor: 'copying' stamp: 'lr 3/28/2007 10:27'!
postCopy
	| previous |
	super postCopy.
	previous := self contents.
	location := nil.
	self contents: previous! !

!MAExternalFileModel methodsFor: 'accessing-dynamic' stamp: 'lr 1/26/2007 18:16'!
readStream
	^ (self directory readOnlyFileNamed: self filename) binary! !

!MAExternalFileModel methodsFor: 'private' stamp: 'lr 9/28/2007 07:44'!
uniqueLocation: aLocationDefinition
	"Finds an unique path to be used and create the necessary sub directories."

	| valid result directory definition |
	valid := false.
	result := Array new: aLocationDefinition size.
	[ valid ] whileFalse: [
		directory := self baseDirectory assureExistence.
		result keysAndValuesDo: [ :index :value |
			definition := aLocationDefinition at: index.
			result at: index put: ((String new: definition first)
				collect: [ :each | definition second atRandom ]).
			directory := directory
				directoryNamed: (result at: index).
			directory exists ifFalse: [
				directory assureExistence.
				valid := true ] ] ].
	^ result! !

!MAExternalFileModel methodsFor: 'accessing-dynamic' stamp: 'lr 1/26/2007 18:16'!
writeStream
	^ (self directory forceNewFileNamed: self filename) binary! !

!GRPharoPlatform methodsFor: '*magritte-pharo-model' stamp: 'jf 9/16/2009 13:39'!
magritteAllSubInstancesOf: aClass do: aBlock
	"Evaluate the aBlock for all instances of aClass and all its subclasses."

	aClass allSubInstancesDo: aBlock! !

!GRPharoPlatform methodsFor: '*magritte-pharo-model' stamp: 'jf 9/16/2009 13:32'!
magritteClassNamed: aString
	"Return the class named aString, nil if the class can't be found."

	^ Smalltalk classNamed: aString! !

!GRPharoPlatform methodsFor: '*magritte-pharo-model' stamp: 'jf 9/15/2009 18:43'!
magritteColorClass
	"Return a Color class"
	
	^ Color! !

!GRPharoPlatform methodsFor: '*magritte-pharo-model' stamp: 'jf 9/17/2009 17:10'!
magritteEvaluatorClassFor: aClass
	"Answer an evaluator class appropriate for evaluating expressions in the 
	context of this class."
	
	^ aClass evaluatorClass! !

!GRPharoPlatform methodsFor: '*magritte-pharo-model' stamp: 'jf 9/16/2009 14:27'!
magritteTimeStampClass
	"Return the platform's TimeStamp class. It is currently assumed that all platforms
	have one, though this may not be correct and could require moving timestamp stuff
	to its own package in order to resolve."
	
	^ TimeStamp! !

!GRPharoPlatform methodsFor: '*magritte-pharo-model' stamp: 'jf 9/16/2009 14:01'!
magritteTimeStampIfAbsent: absentBlock
	"Return the TimeStamp class, or if the platform does not have a TimeStamp
	return the result of evaluating absentBlock."
	
	^ TimeStamp! !

!GRPharoPlatform methodsFor: '*magritte-pharo-model' stamp: 'lr 1/11/2010 23:04'!
magritteUniqueObject
	"Answer a random object, such as a UUID, that is extremely likely to
	be unique over space and time."

	^ ByteArray withAll: UUID new! !

!GRPackage classSide methodsFor: '*magritte-pharo-model' stamp: 'lr 2/16/2010 09:16'!
magrittePharoModel
	^ self new
		name: 'Magritte-Pharo-Model';
		addDependency: 'Magritte-Model';
		url: #magritteUrl;
		yourself! !
MAExternalFileModel initialize!
